@@ -19,8 +19,12 @@
 
 import binascii
 import os
+import sys
 
-from collections import MutableMapping
+if sys.version_info[:2] >= (3, 3):
+    from collections.abc import MutableMapping
+else:
+    from collections import MutableMapping
 from hashlib import sha1
 from hmac import HMAC
 
@@ -152,6 +156,7 @@ class HostKeys(MutableMapping):
                 for e in list(self._entries):
                     if e.key.get_name() == key:
                         self._entries.remove(e)
+                        break
                 else:
                     raise KeyError(key)
 
@@ -302,7 +307,7 @@ class HostKeys(MutableMapping):
             salt = decodebytes(b(salt))
         assert len(salt) == sha1().digest_size
         hmac = HMAC(salt, b(hostname), sha1).digest()
-        hostkey = "|1|%s|%s" % (u(encodebytes(salt)), u(encodebytes(hmac)))
+        hostkey = "|1|{}|{}".format(u(encodebytes(salt)), u(encodebytes(hmac)))
         return hostkey.replace("\n", "")
 
 
@@ -340,10 +345,8 @@ class HostKeyEntry:
         fields = line.split(" ")
         if len(fields) < 3:
             # Bad number of fields
-            log.info(
-                "Not enough fields found in known_hosts in line %s (%r)"
-                % (lineno, line)
-            )
+            msg = "Not enough fields found in known_hosts in line {} ({!r})"
+            log.info(msg.format(lineno, line))
             return None
         fields = fields[:3]
 
@@ -363,7 +366,7 @@ class HostKeyEntry:
             elif keytype == "ssh-ed25519":
                 key = Ed25519Key(data=decodebytes(key))
             else:
-                log.info("Unable to handle key of type %s" % (keytype,))
+                log.info("Unable to handle key of type {}".format(keytype))
                 return None
 
         except binascii.Error as e:
@@ -378,7 +381,7 @@ class HostKeyEntry:
         included.
         """
         if self.valid:
-            return "%s %s %s\n" % (
+            return "{} {} {}\n".format(
                 ",".join(self.hostnames),
                 self.key.get_name(),
                 self.key.get_base64(),
@@ -386,4 +389,4 @@ class HostKeyEntry:
         return None
 
     def __repr__(self):
-        return "<HostKeyEntry %r: %r>" % (self.hostnames, self.key)
+        return "<HostKeyEntry {!r}: {!r}>".format(self.hostnames, self.key)
