@@ -101,9 +101,8 @@ class KexGex(object):
             return self._parse_kexdh_gex_reply(m)
         elif ptype == _MSG_KEXDH_GEX_REQUEST_OLD:
             return self._parse_kexdh_gex_request_old(m)
-        raise SSHException(
-            "KexGex %s asked to handle packet type %d" % self.name, ptype
-        )
+        msg = "KexGex {} asked to handle packet type {:d}"
+        raise SSHException(msg.format(self.name, ptype))
 
     # ...internals...
 
@@ -151,8 +150,9 @@ class KexGex(object):
             raise SSHException("Can't do server-side gex with no modulus pack")
         self.transport._log(
             DEBUG,
-            "Picking p (%d <= %d <= %d bits)"
-            % (minbits, preferredbits, maxbits),
+            "Picking p ({} <= {} <= {} bits)".format(
+                minbits, preferredbits, maxbits
+            ),
         )
         self.g, self.p = pack.get_modulus(minbits, preferredbits, maxbits)
         m = Message()
@@ -176,7 +176,7 @@ class KexGex(object):
         if pack is None:
             raise SSHException("Can't do server-side gex with no modulus pack")
         self.transport._log(
-            DEBUG, "Picking p (~ %d bits)" % (self.preferred_bits,)
+            DEBUG, "Picking p (~ {} bits)".format(self.preferred_bits)
         )
         self.g, self.p = pack.get_modulus(
             self.min_bits, self.preferred_bits, self.max_bits
@@ -197,9 +197,9 @@ class KexGex(object):
         if (bitlen < 1024) or (bitlen > 8192):
             raise SSHException(
                 "Server-generated gex p (don't ask) is out of range "
-                "(%d bits)" % bitlen
+                "({} bits)".format(bitlen)
             )
-        self.transport._log(DEBUG, "Got server p (%d bits)" % bitlen)
+        self.transport._log(DEBUG, "Got server p ({} bits)".format(bitlen))
         self._generate_x()
         # now compute e = g^x mod p
         self.e = pow(self.g, self.x, self.p)
@@ -240,7 +240,9 @@ class KexGex(object):
         H = self.hash_algo(hm.asbytes()).digest()
         self.transport._set_K_H(K, H)
         # sign it
-        sig = self.transport.get_server_key().sign_ssh_data(H)
+        sig = self.transport.get_server_key().sign_ssh_data(
+            H, self.transport.host_key_type
+        )
         # send reply
         m = Message()
         m.add_byte(c_MSG_KEXDH_GEX_REPLY)
