@@ -142,7 +142,7 @@ class AuthTest(unittest.TestCase):
         self.assertTrue(self.event.is_set())
         self.assertTrue(self.ts.is_active())
 
-    def test_1_bad_auth_type(self):
+    def test_bad_auth_type(self):
         """
         verify that we get the right exception when an unsupported auth
         type is requested.
@@ -160,7 +160,7 @@ class AuthTest(unittest.TestCase):
             self.assertEqual(BadAuthenticationType, etype)
             self.assertEqual(["publickey"], evalue.allowed_types)
 
-    def test_2_bad_password(self):
+    def test_bad_password(self):
         """
         verify that a bad password gets the right exception, and that a retry
         with the right password works.
@@ -176,7 +176,7 @@ class AuthTest(unittest.TestCase):
         self.tc.auth_password(username="slowdive", password="pygmalion")
         self.verify_finished()
 
-    def test_3_multipart_auth(self):
+    def test_multipart_auth(self):
         """
         verify that multipart auth works.
         """
@@ -191,7 +191,7 @@ class AuthTest(unittest.TestCase):
         self.assertEqual([], remain)
         self.verify_finished()
 
-    def test_4_interactive_auth(self):
+    def test_interactive_auth(self):
         """
         verify keyboard-interactive auth works.
         """
@@ -210,7 +210,7 @@ class AuthTest(unittest.TestCase):
         self.assertEqual([], remain)
         self.verify_finished()
 
-    def test_5_interactive_auth_fallback(self):
+    def test_interactive_auth_fallback(self):
         """
         verify that a password auth attempt will fallback to "interactive"
         if password auth isn't supported but interactive is.
@@ -221,7 +221,7 @@ class AuthTest(unittest.TestCase):
         self.assertEqual([], remain)
         self.verify_finished()
 
-    def test_6_auth_utf8(self):
+    def test_auth_utf8(self):
         """
         verify that utf-8 encoding happens in authentication.
         """
@@ -231,7 +231,7 @@ class AuthTest(unittest.TestCase):
         self.assertEqual([], remain)
         self.verify_finished()
 
-    def test_7_auth_non_utf8(self):
+    def test_auth_non_utf8(self):
         """
         verify that non-utf-8 encoded passwords can be used for broken
         servers.
@@ -242,7 +242,7 @@ class AuthTest(unittest.TestCase):
         self.assertEqual([], remain)
         self.verify_finished()
 
-    def test_8_auth_gets_disconnected(self):
+    def test_auth_gets_disconnected(self):
         """
         verify that we catch a server disconnecting during auth, and report
         it as an auth failure.
@@ -250,13 +250,13 @@ class AuthTest(unittest.TestCase):
         self.start_server()
         self.tc.connect(hostkey=self.public_host_key)
         try:
-            remain = self.tc.auth_password("bad-server", "hello")
+            self.tc.auth_password("bad-server", "hello")
         except:
             etype, evalue, etb = sys.exc_info()
             self.assertTrue(issubclass(etype, AuthenticationException))
 
     @slow
-    def test_9_auth_non_responsive(self):
+    def test_auth_non_responsive(self):
         """
         verify that authentication times out if server takes to long to
         respond (or never responds).
@@ -265,7 +265,7 @@ class AuthTest(unittest.TestCase):
         self.start_server()
         self.tc.connect()
         try:
-            remain = self.tc.auth_password("unresponsive-server", "hello")
+            self.tc.auth_password("unresponsive-server", "hello")
         except:
             etype, evalue, etb = sys.exc_info()
             self.assertTrue(issubclass(etype, AuthenticationException))
